#include <stdlib.h>
#include "hashMap.h"
#include "structs.h"
#include <string.h>
#include <assert.h>

int stringHash1(char * str)
{
	int i;
	int r = 0;
	for (i = 0; str[i] != '\0'; i++)
		r += str[i];
	return r;
}

int stringHash2(char * str)
{
	int i;
	int r = 0;
	for (i = 0; str[i] != '\0'; i++)
		r += (i+1) * str[i]; /*the difference between 1 and 2 is on this line*/
	return r;
}

void initMap (struct hashMap * ht, int tableSize)
{
	int index;
	if(ht == NULL)
		return;
	ht->table = (hashLink**)malloc(sizeof(hashLink*) * tableSize);
	ht->tableSize = tableSize;
	ht->count = 0;
	for(index = 0; index < tableSize; index++)
		ht->table[index] = NULL;
}

void freeMap (struct hashMap * ht)
{  
	struct hashLink* next;
	struct hashLink* curr;
	int i;
	
	for(i = 0; i < ht->tableSize; i++)
	{
		curr = ht->table[i];
		while(curr != 0)
		{
			next = curr->next;
			free(curr);
			curr = next;
		}
		ht->table[i] = 0;
	}
	free(ht->table);
	ht->tableSize = 0;
	ht->count = 0;
}

void insertMap (struct hashMap * ht, KeyType k, ValueType v)
{
	int index;
	struct hashLink* new = malloc(sizeof(struct hashLink));
	assert(new);
	
	new->next = 0;
	new->key = k;
	new->value = v;
	
	if(containsKey(ht, k))
	{
		removeKey(ht, k);
	}
	
	index = stringHash1(k) % ht->tableSize;
	
	if(index < 0)
	{
		index = ht->tableSize + index;
	}
	if(!ht->table[index])
	{
		ht->table[index] = new;
	}
	else
	{
		struct hashLink* curr = ht->table[index];
		while(curr->next)
		{
			curr = curr->next;
		}
		curr->next = new;
	}
	ht->count++;
}

ValueType* atMap (struct hashMap * ht, KeyType k)
{
	struct hashLink* curr;
	int index = stringHash1(k) % ht->tableSize;
	
	if(index < 0)
	{
		index = index + ht->tableSize;
	}
	assert(containsKey(ht, k));
	curr = ht->table[index];
	
	while(strcmp(curr->key, k) != 0)
	{
		curr = curr->next;
	}
	
	return &curr->value;
}

int containsKey (struct hashMap * ht, KeyType k)
{  
	struct hashLink* curr;
	int index = stringHash1(k) % ht->tableSize;
	if(index < 0)
	{
		index = index + ht->tableSize;
	}
	
	
	curr = ht->table[index];
	
	while(curr)
	{
		if(strcmp(curr->key, k) == 0)
		{
			return 1;
		}
		curr = curr->next;
	}
	return 0;
}

void removeKey (struct hashMap * ht, KeyType k)
{  
	struct hashLink* curr;
	struct hashLink* prev = NULL;
	
	int index = stringHash1(k) % ht->tableSize;
	if(index < 0)
	{
		index = index + ht->tableSize;
	}
	
	curr = ht->table[index];
	while(strcmp(curr->key, k) != 0)
	{
		prev = curr;
		curr = curr->next;
	}
	if(prev)
	{
		prev->next = curr->next;
	}
	else
	{
		ht->table[index] = 0;
	}
	free(curr);
}

int sizeMap (struct hashMap *ht)
{  
	assert(ht);
	return ht->count;
}

int capacityMap(struct hashMap *ht)
{  
	assert(ht);
	return ht->tableSize;
}

int emptyBuckets(struct hashMap *ht)
{  
	int i, numEmpBuck = 0;
	
	for(i = 0; i < ht->tableSize; i++)
	{
		if(!ht->table[i])
		{
			numEmpBuck++;
		}
	}
	return numEmpBuck;
}

float tableLoad(struct hashMap *ht)
{ 
	assert(ht);
	return ht->count / (double) ht->tableSize;
}
