#include <assert.h>
#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include "hashMap.h"

/*
 the getWord function takes a FILE pointer and returns you a string which was
 the next word in the in the file. words are defined (by this function) to be
 characters or numbers seperated by periods, spaces, or newlines.
 
 when there are no more words in the input file this function will return NULL.
 
 this function will malloc some memory for the char* it returns. it is your job
 to free this memory when you no longer need it.
 */
char* getWord(FILE *file); /* prototype */

/****************************************/

int main (int argc, const char * argv[]) 
{
    const char* fileName;
    struct hashMap hashTable;
    int tableSize = 1000;
    clock_t timer;
    int i;
    FILE *inputFile;
    struct hashLink* curr;
    
    if(argc == 2)
    {
		fileName = argv[1];
	}
	else
	{
		fileName = "input.txt";
	}
	
	printf("Opening file: %s\n", fileName);
	
	timer = clock();
	
	initMap(&hashTable, tableSize);
	
	
	inputFile = fopen(fileName, "r");
	if(!inputFile)
	{
		printf("Error, failed to open input file.\n");
	}
	
	while(!feof(inputFile))
	{
		char *word = getWord(inputFile);
		
		if(word)
		{
			if(containsKey(&hashTable, word))
			{
				int *value = atMap(&hashTable, word);
				++(*value);
			}
			else
			{
				insertMap(&hashTable, word, 1);
			}
		}
	}
	
	timer = clock() - timer;
	printf("Concordance took %f seconds\n", (float)timer / (float)CLOCKS_PER_SEC);
	
	
	
	for(i = 0; i < hashTable.tableSize; i++)
	{
		curr = hashTable.table[i];
		while(curr)
		{
			printf("%s:%d\n", curr->key, curr->value);
			curr = curr->next;
		}
	}
		
		freeMap(&hashTable);
		fclose(inputFile);
		return 0;
}

char* getWord(FILE *file)
{
	int length = 0;
	int maxLength = 16;
	char character;
	
	char *word = (char*)malloc(sizeof(char) * maxLength);
	assert(word != NULL);
	while((character = fgetc(file)) != EOF)
	{
		if((length + 1) > maxLength)
		{
			maxLength = maxLength * 2;
			word = (char*)realloc(word, maxLength);
		}
		if((character >= '0' && character <= '9') || 
		   (character >= 'A' && character <= 'Z') ||
		   (character >= 'a' && character <= 'z') || 
		   character == 39)
		   {
			   word[length] = character;
			   length++;
		   }
		   else if(length > 0)
		   {
			   break;
		   }   
	 }
	   
	 if(length == 0)
	 {
	   free(word);
	   return NULL;
	 }
	 word[length] = '\0';
	 return word;
}

